﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls.DragDrop;
using System.Collections.ObjectModel;
using System.Collections;

namespace SilverlightApplication18
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();

            birdsListBox.ItemsSource = GetBirds();
            dinoListBox.ItemsSource = GetDinos();

            //- Wire up drag drop for birds
            RadDragAndDropManager.AddDropQueryHandler(birdsListBox, new EventHandler<DragDropQueryEventArgs>(OnDropQuery));
            RadDragAndDropManager.AddDragQueryHandler(birdsListBox, new EventHandler<DragDropQueryEventArgs>(OnDragQuery));
            RadDragAndDropManager.AddDropInfoHandler(birdsListBox, new EventHandler<DragDropEventArgs>(OnDropInfo));
            RadDragAndDropManager.AddDragInfoHandler(birdsListBox, new EventHandler<DragDropEventArgs>(OnDragInfo));

            //- Wire up drag drop for dinos
            RadDragAndDropManager.AddDropQueryHandler(dinoListBox, new EventHandler<DragDropQueryEventArgs>(OnDropQuery));
            RadDragAndDropManager.AddDragQueryHandler(dinoListBox, new EventHandler<DragDropQueryEventArgs>(OnDragQuery));
            RadDragAndDropManager.AddDropInfoHandler(dinoListBox, new EventHandler<DragDropEventArgs>(OnDropInfo));
            RadDragAndDropManager.AddDragInfoHandler(dinoListBox, new EventHandler<DragDropEventArgs>(OnDragInfo));
        }

        // OnDragQuery event handler
        private void OnDragQuery(object sender, DragDropQueryEventArgs e)
        {
            var listBoxItem = e.Options.Source as ContentControl;

            if (e.Options.Status == DragStatus.DragQuery && listBoxItem != null)
            {
                var sourceControl = e.Options.Source;

                var dragCue = RadDragAndDropManager.GenerateVisualCue(sourceControl);
                dragCue.HorizontalAlignment = HorizontalAlignment.Left;
                dragCue.Content = sourceControl.DataContext;
                dragCue.ContentTemplate = listBoxItem.ContentTemplate;

                e.Options.DragCue = dragCue;
                e.Options.Payload = new DragDropOperation()
                {
                    Payload = sourceControl.DataContext
                };
            }

            e.QueryResult = true;
            e.Handled = true;
        }

        // OnDragInfo event handler
        private void OnDragInfo(object sender, DragDropEventArgs e)
        {
            var listBox = e.Options.Source.FindItemsControlParent();
            var operation = e.Options.Payload as DragDropOperation;

            if (e.Options.Status == DragStatus.DragComplete)
            {
                listBox = e.Options.Source.FindItemsControlParent();
                var itemsSource = listBox.ItemsSource as IList;
                operation = e.Options.Payload as DragDropOperation;
                itemsSource.Remove(operation.Payload);
            }

            StatusText.Text = e.Options.Status.ToString();
        }

        // OnDropQuery event handler
        private void OnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            var destination = e.Options.Destination;
            var operation = e.Options.Payload as DragDropOperation;

            if (e.Options.Status == DragStatus.DropDestinationQuery && destination is System.Windows.Controls.ListBoxItem && operation != null)
            {
                var listBox = destination.FindItemsControlParent() as System.Windows.Controls.ListBox;

                // Cannot place an item relative to itself:
                if (e.Options.Source == e.Options.Destination)
                {
                    return;
                }


                // Check to see if the object being dragged matches the type of objects in the destination list
                Type type = listBox.ItemsSource.GetType().GetProperty("Item").PropertyType;

                if (type != operation.Payload.GetType())
                {
                    // Deny drag drop here some how?
                    e.QueryResult = false;
                    e.Handled = true;

                    return;
                }

                // Get the spatial relation between the destination item and the vis. root:
                var destinationTopLeft = destination.TransformToVisual(null).Transform(new Point());

                // Should the new Item be moved before or after the destination item?:
                bool placeBefore = (e.Options.CurrentDragPoint.Y - destinationTopLeft.Y) < destination.ActualHeight / 2;
                operation.DropPosition = placeBefore ? DropPosition.Before : DropPosition.After;
                e.QueryResult = true;
                e.Handled = true;
            }

            if (e.Options.Status == DragStatus.DropDestinationQuery && destination is System.Windows.Controls.ListBox && operation != null)
            {
                var listBox = destination as System.Windows.Controls.ListBox;
                var itemsCollection = listBox.ItemsSource.Cast<object>();

                // Check to see if the object being dragged matches the type of objects in the destination list
                Type type = listBox.ItemsSource.GetType().GetProperty("Item").PropertyType;

                if (type != operation.Payload.GetType())
                {
                    // Deny drag drop here some how?
                    e.QueryResult = false;
                    e.Handled = true;

                    return;
                }

                // Cannot drop the last or only item of the list box within the same list box:
                if (listBox.ItemsSource != null && (!itemsCollection.Any() || itemsCollection.Last() != operation.Payload))
                {
                    e.QueryResult = true;
                    e.Handled = true;
                }
                else
                {
                    e.QueryResult = false;
                    e.Handled = true;
                }
            }
        }

        // OnDropInfo event handler
        private void OnDropInfo(object sender, DragDropEventArgs e)
        {
            var destination = e.Options.Destination;

            if (e.Options.Status == DragStatus.DropPossible && destination is System.Windows.Controls.ListBoxItem)
            {
                var listBox = destination.FindItemsControlParent() as System.Windows.Controls.ListBox;
                VisualStateManager.GoToState(listBox, "DropPossible", false);

                // Get the DropCueElement:
                var dropCueElement = (VisualTreeHelper.GetChild(listBox, 0) as FrameworkElement).FindName("DropCueElement") as FrameworkElement;
                var operation = e.Options.Payload as DragDropOperation;

                // Get the parent of the destination:
                var visParent = VisualTreeHelper.GetParent(destination) as UIElement;

                // Get the spatial relation between the destination and its parent:
                var destinationStackTopLeft = destination.TransformToVisual(visParent).Transform(new Point());
                var yTranslateValue = operation.DropPosition == DropPosition.Before ? destinationStackTopLeft.Y : destinationStackTopLeft.Y + destination.ActualHeight;
                dropCueElement.RenderTransform = new TranslateTransform()
                {
                    Y = yTranslateValue
                };

                e.Handled = true;
            }

            if (e.Options.Status == DragStatus.DropPossible && destination is System.Windows.Controls.ListBox)
            {
                var listBox = destination as System.Windows.Controls.ListBox;
                VisualStateManager.GoToState(listBox, "DropPossible", false);

                // Get the DropCueElement:
                var dropCueElement = (VisualTreeHelper.GetChild(listBox, 0) as FrameworkElement).FindName("DropCueElement") as FrameworkElement;
                var operation = e.Options.Payload as DragDropOperation;

                // Get the size of the items:
                var itemsPresenter = listBox.GetTemplateChild<FrameworkElement>("ItemsPresenterElement");
                var panel = VisualTreeHelper.GetChild(itemsPresenter, 0) as Panel;

                if (panel != null)
                {
                    var yTranslateValue = panel.ActualHeight;
                    dropCueElement.RenderTransform = new TranslateTransform()
                    {
                        Y = yTranslateValue
                    };
                }
            }

            // Hide the DropCue:
            if (e.Options.Status == DragStatus.DropImpossible || e.Options.Status == DragStatus.DropCancel || e.Options.Status == DragStatus.DropComplete)
            {
                var listBox = destination as System.Windows.Controls.ListBox;
                if (listBox == null)
                {
                    listBox = e.Options.Destination.FindItemsControlParent() as System.Windows.Controls.ListBox;
                }

                VisualStateManager.GoToState(listBox, "DropImpossible", false);
            }

            // Place the item:
            if (e.Options.Status == DragStatus.DropComplete && destination is System.Windows.Controls.ListBoxItem)
            {
                var listBox = e.Options.Destination.FindItemsControlParent() as System.Windows.Controls.ListBox;
                var itemsSource = listBox.ItemsSource as IList;
                var destinationIndex = itemsSource.IndexOf(e.Options.Destination.DataContext);
                var operation = e.Options.Payload as DragDropOperation;
                var insertIndex = operation.DropPosition == DropPosition.Before ? destinationIndex : destinationIndex + 1;

                itemsSource.Insert(insertIndex, operation.Payload);
                listBox.Dispatcher.BeginInvoke(() =>
                {
                    listBox.SelectedIndex = insertIndex;
                });
            }

            if (e.Options.Status == DragStatus.DropComplete && destination is System.Windows.Controls.ListBox)
            {
                var listBox = destination as System.Windows.Controls.ListBox;
                var itemsSource = listBox.ItemsSource as IList;
                var operation = e.Options.Payload as DragDropOperation;

                itemsSource.Add(operation.Payload);
                listBox.Dispatcher.BeginInvoke(() =>
                {
                    listBox.SelectedIndex = itemsSource.Count - 1;
                });
            }
        }



        public static ObservableCollection<Bird> GetBirds()
        {
            ObservableCollection<Bird> items = new ObservableCollection<Bird>();
            items.Add(new Bird() { Name = "Meadow Lark" });
            items.Add(new Bird() { Name = "Crow" });
            items.Add(new Bird() { Name = "Sparrow" });
            items.Add(new Bird() { Name = "Canary" });
            items.Add(new Bird() { Name = "Robin" });
            items.Add(new Bird() { Name = "Oriole" });
            items.Add(new Bird() { Name = "Pheasant" });
            items.Add(new Bird() { Name = "Magpie" });
            items.Add(new Bird() { Name = "Cardinal" });
            items.Add(new Bird() { Name = "Parrot" });
            items.Add(new Bird() { Name = "Finch" });
            items.Add(new Bird() { Name = "Raven" });
            items.Add(new Bird() { Name = "French Hen" });
            items.Add(new Bird() { Name = "Duck" });
            items.Add(new Bird() { Name = "Cornish Game Hen" });
            items.Add(new Bird() { Name = "Chicken" });
            items.Add(new Bird() { Name = "Ostrich" });
            items.Add(new Bird() { Name = "Roadrunner" });

            return items;
        }

        public static ObservableCollection<Dinosaur> GetDinos()
        {
            ObservableCollection<Dinosaur> items = new ObservableCollection<Dinosaur>();
            items.Add(new Dinosaur() { Name = "T-Rex" });
            items.Add(new Dinosaur() { Name = "Triceritops" });
            items.Add(new Dinosaur() { Name = "Brontosarus" });
            items.Add(new Dinosaur() { Name = "Velociraptor" });
            items.Add(new Dinosaur() { Name = "Alligator" });
            items.Add(new Dinosaur() { Name = "Brachiasaurus" });
            items.Add(new Dinosaur() { Name = "Stegasauraus" });
            items.Add(new Dinosaur() { Name = "Pteridactyl" });

            return items;
        }
    }

    public enum DropPosition
    {
        After,
        Before,
        Inside
    }

    public class DragDropOperation
    {
        public DropPosition DropPosition
        {
            get;
            set;
        }
        public object Payload
        {
            get;
            set;
        }
    }

    public class Bird
    {
        public string Name { get; set; }
    }

    public class Dinosaur
    {
        public string Name { get; set; }
    }
}
